﻿using DoorsExpanded;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace AdvancedDoorsDE
{
    public class CompProperties_DoorExpandedFuckHardcodingThings : CompProperties_DoorExpanded
    {
        public override void ResolveReferences(ThingDef parentDef)
        {
            base.ResolveReferences(parentDef);
            parentDef.thingClass = typeof(Building_AdvancedDoorDE);
        }
    }

    public class Building_AdvancedDoorDE : Building_DoorExpanded
    {
        private Graphic graphicIntRight;
        public Graphic DefaultGraphicRight
        {
            get
            {
                if (graphicIntRight == null)
                {
                    var otherGraphicData = new GraphicData
                    {
                        texPath = props.doorAsync.texPath,
                        graphicClass = props.doorAsync.graphicClass,
                        shadowData = props.doorAsync.shadowData,
                        shaderType = props.doorAsync.shaderType,
                        shaderParameters = props.doorAsync.shaderParameters,
                        onGroundRandomRotateAngle = props.doorAsync.onGroundRandomRotateAngle,
                        linkType = props.doorAsync.linkType,
                        linkFlags = props.doorAsync.linkFlags,
                        flipExtraRotation = props.doorAsync.flipExtraRotation,
                        drawSize = props.doorAsync.drawSize,
                        drawRotated = !def.graphicData.drawRotated,
                        drawOffsetWest = props.doorAsync.drawOffsetWest,
                        drawOffsetSouth = props.doorAsync.drawOffsetSouth,
                        drawOffsetNorth = props.doorAsync.drawOffsetNorth,
                        drawOffsetEast = props.doorAsync.drawOffsetEast,
                        drawOffset = props.doorAsync.drawOffset,
                        damageData = props.doorAsync.damageData,
                        colorTwo = props.doorAsync.colorTwo,
                        color = props.doorAsync.color,
                        allowFlip = props.doorAsync.allowFlip
                    };
                    graphicIntRight = otherGraphicData.GraphicColoredFor(this);
                }
                return graphicIntRight;
            }
        }
        public Graphic GraphicRight => DefaultGraphicRight;

        public override void Draw()
        {
            var ticksToOpenNow = TicksToOpenNow;
            var percentOpen = ticksToOpenNow == 0 ? 1f : Mathf.Clamp01((float)ticksSinceOpen / ticksToOpenNow);

            var rotation = DoorRotationAt(def, props, Position, Rotation, Map);
            Rotation = rotation;

            var drawPos = DrawPos;
            drawPos.y = AltitudeLayer.DoorMoveable.AltitudeFor();
            for (var i = 0; i < 2; i++)
            {
                var flipped = i != 0;
                var material = (!flipped && props.doorAsync is GraphicData doorAsyncGraphic)
                    ? GraphicRight.MatAt(rotation)
                    : Graphic.MatAt(rotation);

                Draw(def, props, material, drawPos, rotation, percentOpen, flipped,
                    i == 0 && MoreDebugViewSettings.writeDoors ? debugDrawVectors : null);
                if (props.singleDoor)
                    break;
            }

            if (props.doorFrame is GraphicData)
            {
                DrawFrameParams(def, props, drawPos, rotation, false, out var fMesh, out var fMatrix);
                Graphics.DrawMesh(fMesh, fMatrix, props.doorFrame.GraphicColoredFor(this).MatAt(rotation), 0);

                if (props.doorFrameSplit is GraphicData)
                {
                    DrawFrameParams(def, props, drawPos, rotation, true, out fMesh, out fMatrix);
                    Graphics.DrawMesh(fMesh, fMatrix, props.doorFrameSplit.GraphicColoredFor(this).MatAt(rotation), 0);
                }
            }

            Comps_PostDraw();
        }

        internal static new void Draw(ThingDef def, CompProperties_DoorExpanded props,
            Material material, Vector3 drawPos, Rot4 rotation, float percentOpen,
            bool flipped, DebugDrawVectors drawVectors = null)
        {
            Mesh mesh;
            Quaternion rotQuat;
            Vector3 offsetVector, scaleVector;
            switch (props.doorType)
            {
                // There's no difference between Stretch and StretchVertical except for stretchOpenSize's default value.
                case DoorType.Stretch:
                case DoorType.StretchVertical:
                    DrawStretchParams(def, props, rotation, percentOpen, flipped,
                        out mesh, out rotQuat, out offsetVector, out scaleVector);
                    break;
                case DoorType.DoubleSwing:
                    // TODO: Should drawPos.y be set to Mathf.Max(drawPos.y, AltitudeLayer.BuildingOnTop.AltitudeFor())
                    // since AltitudeLayer.DoorMoveable is only used to hide sliding doors behind adjacent walls?
                    DrawDoubleSwingParams(def, props, drawPos, rotation, percentOpen, flipped,
                        out mesh, out rotQuat, out offsetVector, out scaleVector);
                    break;
                default:
                    DrawStandardParams(def, props, rotation, percentOpen, flipped,
                        out mesh, out rotQuat, out offsetVector, out scaleVector);
                    break;
            }
            var graphicVector = drawPos + offsetVector;
            var matrix = Matrix4x4.TRS(graphicVector, rotQuat, scaleVector);
            Graphics.DrawMesh(mesh, matrix, material, layer: 0);

            if (drawVectors != null)
            {
                drawVectors.percentOpen = percentOpen;
                drawVectors.offsetVector = offsetVector;
                drawVectors.scaleVector = scaleVector;
                drawVectors.graphicVector = graphicVector;
            }
        }

        private static new void DrawStandardParams(ThingDef def, CompProperties_DoorExpanded props,
    Rot4 rotation, float percentOpen, bool flipped, out Mesh mesh, out Quaternion rotQuat,
    out Vector3 offsetVector, out Vector3 scaleVector)
        {
            var verticalRotation = rotation.IsHorizontal;
            if (!flipped)
            {
                offsetVector = new Vector3(-1f, 0f, 0f);
                mesh = MeshPool.plane10;
            }
            else
            {
                offsetVector = new Vector3(1f, 0f, 0f);
                mesh = MeshPool.plane10;
            }

            rotQuat = rotation.AsQuat;
            offsetVector = rotQuat * offsetVector;

            var offsetMod = (VisualDoorOffsetStart + props.doorOpenMultiplier * percentOpen) * def.Size.x;
            offsetVector *= offsetMod;

            var drawSize= props.doorAsync.drawSize;
            var persMod = verticalRotation && props.fixedPerspective ? 2f : 1f;
            scaleVector = new Vector3(drawSize.x * persMod, 1f, drawSize.y * persMod);
        }
    }
}
